#!/bin/sh
#spc1.sh A B
#A : SPC1 TEST PHASE
#B : SPC1 IOPS
cmd=$1
iops=$2

date=$(date '+%Y-%m-%d_%H:%M:%S')
date_time=$(date '+%H:%M:%S')
spc_home="/root/spc"
spc_master=$spc_home"/5host.HST"
log_home="/root/Logs"
target_ip="10.144.144.201 10.144.144.202 10.144.144.203 10.144.144.204 10.144.144.205 10.144.144.206 10.144.144.207 10.144.144.208 10.144.144.209 10.144.144.210"
host_ip="10.144.144.101 10.144.144.102 10.144.144.103 10.144.144.104 10.144.144.105"

for ip in $target_ip;
do
	ping $ip -c 3
	echo ---------------------------------------------------------------
done

for ip in $host_ip;
do
	ping $ip -c 3
	echo ---------------------------------------------------------------
done

echo "ready?"
read 

mkdir -p $log_home"/"$date
log_dir=$log_home"/"$date
sys_log=$log_dir"/inventory_start.out"
sys_log_end=$log_dir"/inventory_end.out"
mdadm_log=$log_dir"/mdadm.log"
stat_log_dir=$log_dir"/stat"
parm_log=$log_dir"/parm.log"
mkdir -p $stat_log_dir

cp /root/mgmt_script/spc1.sh $log_dir"/spc1-backup.sh"
cp $spc_home"/5host.HST" $log_dir"/5host.HST"
cp $spc_home"/SPC1_METRICS" $log_dir"/SPC1_METRICS"

echo "1. Get host information."

cat /etc/os-release >> $sys_log
echo -e "==============================================\n" >> $sys_log
uname -r >> $sys_log
echo -e "==============================================\n" >> $sys_log
nvme list >> $sys_log
echo -e "==============================================\n" >> $sys_log
lsblk -b >> $sys_log
echo -e "==============================================\n" >> $sys_log
fdisk -l >> $sys_log
echo -e "==============================================\n" >> $sys_log
#cat /proc/mdstat >> $sys_log
#echo -e "==============================================\n" >> $sys_log
pvs --units G >> $sys_log
echo -e "==============================================\n" >> $sys_log
vgs --units G >> $sys_log
echo -e "==============================================\n" >> $sys_log
lvs --units G -v --segment >> $sys_log
echo -e "==============================================\n" >> $sys_log

echo "2. Get NVMe information.(smartctl)"

for ip in $target_ip;
do
	echo ====================$ip================= >> $sys_log
	dn=$(ssh root@$ip 'ls /dev/nvme?' | wc -l)
	#dn=`expr $dn - 1`
	for ((i=0;i<$dn;i++));
	do
		echo +++++++++++++++++++++/dev/nvme$i++++++++++++++++++ >> $sys_log
        ssh root@$ip smartctl -a /dev/nvme$i >> $sys_log
	done
done

echo "3. Get Storage-RAID information."

for ip in $target_ip;
do
	echo ====================$ip================= >> $mdadm_log
	dn=$(ssh root@$ip 'ls /dev/md?' | wc -l)
	for ((i=1;i<=$dn;i++));
	do	
		echo +++++++++++++++++++++/dev/md$i++++++++++++++++++ >> $mdadm_log
	    ssh root@$ip mdadm -D /dev/md$i >> $mdadm_log
	done

	echo +++++++++++++++++++++/etc/mdadm.conf++++++++++++++++++ >> $mdadm_log
        ssh root@$ip cat /etc/mdadm.conf >> $mdadm_log
done

echo "4. Get host disk parameter."

for ip in $host_ip;
do
	devs=$(ssh root@$ip 'ls /dev/nvme?n?' | awk -F '/' '{print $3}')

	echo =====================$ip===================== >> $parm_log

	for dev in $devs;
	do
		echo +++++++++++++++/dev/$dev+++++++++++++++ >> $parm_log
		#devices configuration setup
		echo -n 'nr_requests : ' >> $parm_log
		ssh root@$ip cat /sys/block/$dev/queue/nr_requests >> $parm_log
		echo -n 'scheduler : ' >> $parm_log
		ssh root@$ip cat /sys/block/$dev/queue/scheduler >> $parm_log
		echo -n 'max_sectors_kb : ' >> $parm_log
		ssh root@$ip cat /sys/block/$dev/queue/max_sectors_kb >> $parm_log
		echo -n 'add_random : ' >> $parm_log
		ssh root@$ip cat /sys/block/$dev/queue/add_random >> $parm_log
		echo -n 'nomerges : ' >> $parm_log
		ssh root@$ip cat /sys/block/$dev/queue/nomerges >> $parm_log
	done
	#OS configuration setup
	echo '+++++++++++++++system parameter+++++++++++++++' >> $parm_log
	echo -n 'aio-max-nr : '	>> $parm_log
	echo '----------------------------------------------' >> $parm_log
	ssh root@$ip cat /proc/sys/fs/aio-max-nr >> $parm_log
	echo 'ulimit -a' >> $parm_log
	ssh root@$ip ulimit -a >> $parm_log
	echo '----------------------------------------------' >> $parm_log
	echo 'ulimit -aH' >> $parm_log
	ssh root@$ip ulimit -aH >> $parm_log
done

echo "5. Get storage NVMe parameter."

for ip in $target_ip;
do
	devs=$(ssh root@$ip 'ls /dev/nvme?n?' | awk -F '/' '{print $3}')

	echo =====================$ip===================== >> $parm_log

	for dev in $devs;
	do
		echo +++++++++++++++/dev/$dev+++++++++++++++ >> $parm_log
		#devices configuration setup
		echo -n 'nr_requests : ' >> $parm_log
		ssh root@$ip cat /sys/block/$dev/queue/nr_requests >> $parm_log
		echo -n 'scheduler : ' >> $parm_log
		ssh root@$ip cat /sys/block/$dev/queue/scheduler >> $parm_log
		echo -n 'max_sectors_kb : ' >> $parm_log
		ssh root@$ip cat /sys/block/$dev/queue/max_sectors_kb >> $parm_log
		echo -n 'add_random : ' >> $parm_log
		ssh root@$ip cat /sys/block/$dev/queue/add_random >> $parm_log
		echo -n 'nomerges : ' >> $parm_log
		ssh root@$ip cat /sys/block/$dev/queue/nomerges >> $parm_log
	done
	#OS configuration setup
	echo '+++++++++++++++system parameter+++++++++++++++' >> $parm_log
	echo -n 'aio-max-nr : '	>> $parm_log
	ssh root@$ip cat /proc/sys/fs/aio-max-nr >> $parm_log
	echo '----------------------------------------------' >> $parm_log
	echo 'ulimit -a' >> $parm_log
	ssh root@$ip ulimit -a >> $parm_log
	echo '----------------------------------------------' >> $parm_log
	echo 'ulimit -aH' >> $parm_log
	ssh root@$ip ulimit -aH >> $parm_log
done

echo "6. Get storage RAID-Volume parameter."

for ip in $target_ip;
do
	devs=$(ssh root@$ip 'ls /dev/md?' | awk -F '/' '{print $3}')

	echo =====================$ip===================== >> $parm_log

	for dev in $devs;
	do
		echo +++++++++++++++/dev/$dev+++++++++++++++ >> $parm_log
		#devices configuration setup
		echo -n 'nr_requests : ' >> $parm_log
		ssh root@$ip cat /sys/block/$dev/queue/nr_requests >> $parm_log
		echo -n 'scheduler : ' >> $parm_log
		ssh root@$ip cat /sys/block/$dev/queue/scheduler >> $parm_log
		echo -n 'max_sectors_kb : ' >> $parm_log
		ssh root@$ip cat /sys/block/$dev/queue/max_sectors_kb >> $parm_log
		echo -n 'add_random : ' >> $parm_log
		ssh root@$ip cat /sys/block/$dev/queue/add_random >> $parm_log
		echo -n 'nomerges : ' >> $parm_log
		ssh root@$ip cat /sys/block/$dev/queue/nomerges >> $parm_log
	done
done

echo "7. Start monitoring system."

#storage
for ip in $target_ip;
do
	ssh root@$ip '/root/mgmt_script/kill.sh vmstat'
	ssh root@$ip '/root/mgmt_script/kill.sh iostat'
	ssh root@$ip 'rm -rf /tmp/*stat.log'
	ssh root@$ip 'vmstat -t 10 > /tmp/vmstat.log &'
	ssh root@$ip 'iostat -mtx 10 > /tmp/iostat.log &'
done
		
#host
for ip in $host_ip;
do
	ssh root@$ip '/root/mgmt_script/kill.sh vmstat'
	ssh root@$ip '/root/mgmt_script/kill.sh iostat'
	ssh root@$ip 'rm -rf /tmp/*stat.log'
	ssh root@$ip 'vmstat -t 10 > /tmp/vmstat.log &'
	ssh root@$ip 'iostat -mtx 10 > /tmp/iostat.log &'
done

echo "8. Start SPC-1 Test."
sleep 1
echo -n "3.."
sleep 1
echo -n "2.."
sleep 1
echo -n "1.."
sleep 1
echo "Start!!"
sleep 2

## running spc1
/root/spc/spc1 -run SPC1_INIT -iops 15000 -master $spc_master -output $log_dir
/root/spc/spc1 -run SPC1_VERIFY -iops 100 -master $spc_master -output $log_dir
/root/spc/spc1 -run SPC1_METRICS -iops 2410000 -master $spc_master -output $log_dir
/root/spc/spc1 -run SPC1_VERIFY -iops 100 -master $spc_master -output $log_dir
sleep 120

echo "9. Get system monitoring logs."
#storage
for ip in $target_ip;
do
	ssh root@$ip '/root/mgmt_script/kill.sh vmstat'
	ssh root@$ip '/root/mgmt_script/kill.sh iostat'
	scp root@$ip:/tmp/vmstat.log $stat_log_dir"/"$ip"_vmstat.log"
	scp root@$ip:/tmp/iostat.log $stat_log_dir"/"$ip"_iostat.log"
done
		
#host
for ip in $host_ip;
do
	ssh root@$ip '/root/mgmt_script/kill.sh vmstat'
	ssh root@$ip '/root/mgmt_script/kill.sh iostat'
	scp root@$ip:/tmp/vmstat.log $stat_log_dir"/"$ip"_vmstat.log"
	scp root@$ip:/tmp/iostat.log $stat_log_dir"/"$ip"_iostat.log"
done

echo "Stop"
read
/root/spc/spc1 -run SPC1_PERSIST_1 -iops 602500 -master $spc_master -output $log_dir

echo "reboot"
read
/root/spc/spc1 -run SPC1_PERSIST_2 -iops 602500 -master $spc_master -output $log_dir

## finish spc1
nvme list >> $sys_log_end

echo -e "==============================================\n" >> $sys_log_end
lsblk -b >> $sys_log_end
echo -e "==============================================\n" >> $sys_log_end
fdisk -l >> $sys_log_end
echo -e "==============================================\n" >> $sys_log_end
#cat /proc/mdstat >> $sys_log_end
#echo -e "==============================================\n" >> $sys_log_end
pvs --units G >> $sys_log_end
echo -e "==============================================\n" >> $sys_log_end
vgs --units G >> $sys_log_end
echo -e "==============================================\n" >> $sys_log_end
lvs --units G -v --segment >> $sys_log_end
echo -e "==============================================\n" >> $sys_log_end

for ip in $target_ip;
do
	echo ====================$ip================= >> $sys_log_end
	dn=$(ssh root@$ip 'ls /dev/nvme?' | wc -l)
	#dn=`expr $dn - 1`
	for ((i=0;i<=$dn;i++));
	do
		echo +++++++++++++++++++++/dev/nvme$i++++++++++++++++++ >> $sys_log_end
        ssh root@$ip smartctl -a /dev/nvme$i >> $sys_log_end
	done
done

for ip in $target_ip
do
	echo =====================$ip=====================
	ssh root@$ip 'rm -rf /tmp/*stat.log'
done

for ip in $host_ip
do
	echo =====================$ip=====================
	ssh root@$ip 'rm -rf /tmp/*stat.log'
done
